import {Component, Input, OnInit} from '@angular/core';
import {Gamesystem} from "../../game-model/gamesystems/Gamesystem";
import {State} from "../../game-model/gamesystems/State";
import {Transition} from "../../game-model/gamesystems/Transition";
import {ProductGamesystem} from "../../game-model/gamesystems/ProductGamesystem";
import {FlatTreeControl} from "@angular/cdk/tree";
import {
  MatTree,
  MatTreeFlatDataSource,
  MatTreeFlattener,
  MatTreeNode, MatTreeNodeDef,
  MatTreeNodePadding, MatTreeNodeToggle
} from "@angular/material/tree";
import {MatIcon} from "@angular/material/icon";
import {MatIconButton} from "@angular/material/button";
import {SimpleGamesystem} from "../../game-model/gamesystems/SimpleGamesystem";
import {GameModel} from "../../game-model/GameModel";


interface FlatNode {
  expandable: boolean,
  name: string,
  level: number
}
@Component({
  selector: 'app-gamescript-overview',
  templateUrl: './gamescript-overview.component.html',
  styleUrl: './gamescript-overview.component.scss'
})
export class GamescriptOverviewComponent implements OnInit {

  @Input('gameModel') gameModel: GameModel | undefined

  ngOnInit() {
    this.dataSource.data = this.gameModel!.gamesystems;
  }

  private _transformer = (node: Gamesystem<State<any>, Transition<any>>, level: number) => {
    return {
      expandable: this.isProductGamesystem(node) && !!(node as ProductGamesystem).innerGamesystems && (node as ProductGamesystem).innerGamesystems.length > 0,
      name: node.componentName,
      level: level
    }
  }

  treeControl = new FlatTreeControl<FlatNode>(
    node => node.level,
    node => node.expandable
  )

  treeFlattener = new MatTreeFlattener(
    this._transformer,
    node => node.level,
    node => node.expandable,
    node => this.isSimpleGamesystem(node)? []: (node as ProductGamesystem).innerGamesystems
  );

  dataSource = new MatTreeFlatDataSource(this.treeControl, this.treeFlattener);

  constructor() {

  }

  hasChild = (_: number, node: FlatNode) => node.expandable;

  isSimpleGamesystem(gamesystem: Gamesystem<State<any>, Transition<any>>) {
    return gamesystem instanceof SimpleGamesystem;
  }

  isProductGamesystem(gamesystem: Gamesystem<State<any>, Transition<any>>) {
    return gamesystem instanceof ProductGamesystem;
  }
}
